package net.es.oscars.rmi.aaa;

import java.rmi.RemoteException;
import java.util.*;

import org.apache.log4j.*;
import org.hibernate.Session;
import org.hibernate.Hibernate;

import net.es.oscars.aaa.*;
import net.es.oscars.rmi.model.*;

public class UserModelRmiHandler extends ModelRmiHandlerImpl {
    private Logger log;
    private AAACore core = AAACore.getInstance();

    public UserModelRmiHandler() {
        this.log = Logger.getLogger(this.getClass());
    }

    public HashMap<String, Object> list(HashMap<String, Object> parameters)
            throws RemoteException {

        this.log.debug("listUsers.start");
        Session aaa = core.getAaaSession();
        HashMap<String, Object> result = new HashMap<String, Object>();
        String listType = (String) parameters.get("listType");
        if (listType == null) {
            listType = "plain";
        }
        List<User> users = new ArrayList<User>();

        try {
            aaa.beginTransaction();
            if (listType.equals("plain")) {
                UserDAO userDAO = new UserDAO(core.getAaaDbName());
                users = userDAO.list();

            } else if (listType.equals("byAttr")) {
                String attributeName = (String) parameters.get("attributeName");
                if (attributeName == null) {
                    throw new AAAException("attributeName not specified");
                }
                UserAttributeDAO dao = new UserAttributeDAO(core.getAaaDbName());
                users = dao.getUsersByAttribute(attributeName);
            } else if (listType.equals("single")) {
                String userName = (String) parameters.get("username");
                if (userName == null) {
                    throw new AAAException("username not specified");
                }
                UserManager mgr = core.getUserManager();
                User user = mgr.query(userName);
                users = new ArrayList<User>();
                users.add(user);
            } else {
                throw new AAAException("unknown listType");
            }
            for (User user : users) {
                Hibernate.initialize(user);
                Hibernate.initialize(user.getInstitution());
                Hibernate.initialize(user.getInstitution().getUsers());
            }
            aaa.getTransaction().commit();
        } catch (AAAException ex) {
            this.log.info(ex);
            aaa.getTransaction().rollback();
            throw new RemoteException(ex.getMessage(),ex);
        }
        result.put("users", users);
        this.log.debug("listUsers.end");
        return result;
    }

    public HashMap<String, Object> add(HashMap<String, Object> parameters)
            throws RemoteException {

        this.log.info("addUser.start");
        HashMap<String, Object> result = new HashMap<String, Object>();
        ArrayList<String> addRoles =
            (ArrayList<String>) parameters.get("addRoles");
        User user = (User) parameters.get("user");
        if (user == null) {
            throw new RemoteException("User not set");
        } else if (addRoles == null) {
            throw new RemoteException("Roles not set");
        }

        Session aaa = core.getAaaSession();
        aaa.beginTransaction();
        UserManager mgr = core.getUserManager();
        try {
            mgr.create(user, addRoles);
            aaa.getTransaction().commit();
        } catch (AAAException ex) {
            this.log.info("Caught AAAException " + ex.getMessage());
            aaa.getTransaction().rollback();
            throw new RemoteException(ex.getMessage(), ex);
         }
        this.log.info("added user: " + user.getLogin());
        this.log.info("addUser.end");
        return result;
    }

    public HashMap<String, Object> modify(HashMap<String, Object> parameters)
            throws RemoteException {

        this.log.info("modifyUser.start");
        HashMap<String, Object> result = new HashMap<String, Object>();
        ArrayList<String> newRoles =
            (ArrayList<String>) parameters.get("newRoles");
        ArrayList<String> curRoles =
            (ArrayList<String>) parameters.get("curRoles");

        // transient
        User modifiedUser = (User) parameters.get("user");
        Boolean setPassword = (Boolean) parameters.get("setPassword");
        if (setPassword == null) {
            setPassword = false;
        }
        if (modifiedUser == null) {
            throw new RemoteException("User not set");
        } else if (newRoles == null) {
            throw new RemoteException("Roles not set");
        }
        Session aaa = core.getAaaSession();
        try {
            aaa.beginTransaction();
            UserDAO userDAO = new UserDAO(core.getAaaDbName());
            User user = userDAO.queryByParam("login", modifiedUser.getLogin());
            if (user == null) {
                throw new AAAException("User " + modifiedUser.getLogin() +
                                          " does not exist");
            }
            InstitutionDAO institutionDAO =
                new InstitutionDAO(core.getAaaDbName());
            Institution modifiedInst = institutionDAO.queryByParam(
                               "name", modifiedUser.getInstitution().getName());
            if (modifiedInst == null) {
                throw new AAAException("Institution " +
                                       modifiedUser.getInstitution().getName() +
                                       " not found!");
            }
            user.setCertIssuer(modifiedUser.getCertIssuer());
            user.setCertSubject(modifiedUser.getCertSubject());
            user.setLastName(modifiedUser.getLastName());
            user.setFirstName(modifiedUser.getFirstName());
            user.setEmailPrimary(modifiedUser.getEmailPrimary());
            user.setPhonePrimary(modifiedUser.getPhonePrimary());
            if (setPassword) {
                user.setPassword(modifiedUser.getPassword());
            }
            user.setDescription(modifiedUser.getDescription());
            user.setEmailSecondary(modifiedUser.getEmailSecondary());
            user.setPhoneSecondary(modifiedUser.getPhoneSecondary());
            if (user.getInstitution().getName() !=
                  modifiedUser.getInstitution().getName()) {
                user.setInstitution(modifiedInst);
            }
            UserManager mgr = core.getUserManager();
            UserAttributeDAO userAttrDAO =
                new UserAttributeDAO(core.getAaaDbName());
            mgr.update(user, setPassword);
            for (String newRoleItem: newRoles) {
                if (!curRoles.contains(newRoleItem)) {
                    this.log.info("adding user attribute "+ newRoleItem + " for user " + user.getLogin());
                    this.addUserAttribute(newRoleItem, user);
                }
            }
            for (String curRoleItem: curRoles){
               if (!newRoles.contains(curRoleItem)) {
                    userAttrDAO.remove(user.getLogin(), curRoleItem);
                }
            }
            aaa.getTransaction().commit();
        } catch (AAAException ex) {
            this.log.info(ex);
            aaa.getTransaction().rollback();
            throw new RemoteException(ex.getMessage(),ex);
        } finally {

        }
        this.log.info("modifyUser.end");
        return result;
    }

    public HashMap<String, Object> delete(HashMap<String, Object> parameters) throws RemoteException {
        this.log.info("deleteUser.start");
        Session aaa = core.getAaaSession();
        aaa.beginTransaction();
        String username = (String) parameters.get("username");
        UserManager mgr = core.getUserManager();
        try {
            mgr.remove(username);
            aaa.getTransaction().commit();
        } catch (AAAException ex) {
            this.log.info(ex);
            aaa.getTransaction().rollback();
            throw new RemoteException(ex.getMessage(),ex);
        }
        HashMap<String, Object> result = new HashMap<String, Object>();
        this.log.info("deleted user: "+ username);
        this.log.info("deleteUser.end");
        return result;
    }

    public HashMap<String, Object> find(HashMap<String, Object> parameters)
            throws RemoteException {

        this.log.debug("findUser.start");
        HashMap<String, Object> result = new HashMap<String, Object>();
        User user = null;
        String findBy = (String) parameters.get("findBy");
        Session aaa = core.getAaaSession();
        String errMessage = null;
        RemoteException remEx = null;
        try {
            aaa.beginTransaction();
            if (findBy == null || findBy.equals("id")) {
                UserDAO userDAO = new UserDAO(core.getAaaDbName());
                Integer id = (Integer) parameters.get("id");
                if (id == null) {
                    throw new AAAException("Unknown id");
                }
                user = userDAO.findById(id, false);

            } else if (findBy.equals("username")) {
                String username = (String) parameters.get("username");
                if (username == null) {
                    throw new AAAException("Unknown id");
                }
                UserManager mgr = core.getUserManager();
                user = mgr.query(username);

            } else {
                throw new AAAException("Unknown findBy");
            }
            if (user != null) {
                this.log.debug("findUser.found:"+user.getLogin());
                Hibernate.initialize(user);
                Hibernate.initialize(user.getInstitution());
                Hibernate.initialize(user.getInstitution().getUsers());
                result.put("user", user);
            } else {
                throw new AAAException("User not found");
            }
            aaa.getTransaction().commit();
        } catch (AAAException ex) {
            aaa.getTransaction().rollback();
            this.log.info(ex);
            throw new RemoteException(ex.getMessage(),ex);
        }
        this.log.debug("findUser.end");
        return result;
    }

    private void addUserAttribute(String attrName, User user){

        UserAttributeDAO userAttrDAO = new UserAttributeDAO(core.getAaaDbName());
        AttributeDAO attrDAO = new AttributeDAO(core.getAaaDbName());
        UserAttribute userAttr = new UserAttribute();
        Attribute attr = attrDAO.queryByParam("name", attrName);
        userAttr.setAttribute(attr);
        userAttr.setUser(user);
        userAttrDAO.create(userAttr);
    }
}
